﻿using System;
using System.IO;
using System.ServiceModel.Channels;
using System.Xml;

class CustomMessageEncoder : MessageEncoder
{
	readonly CustomMessageEncoderBindingElement bindingElement;
	readonly MessageEncoder innerEncoder;

	public CustomMessageEncoder(CustomMessageEncoderBindingElement bindingElement, MessageEncoder innerEncoder)
	{
		this.bindingElement = bindingElement;
		this.innerEncoder = innerEncoder;
	}

	public override string ContentType
	{
		get { return this.innerEncoder.ContentType; }
	}

	public override string MediaType
	{
		get { return this.innerEncoder.MediaType; }
	}

	public override MessageVersion MessageVersion
	{
		get { return this.innerEncoder.MessageVersion; }
	}

	public override Message ReadMessage(ArraySegment<byte> buffer, BufferManager bufferManager, string contentType)
	{
		byte[] msgContents = new byte[buffer.Count];
		Array.Copy(buffer.Array, buffer.Offset, msgContents, 0, msgContents.Length);
		bufferManager.ReturnBuffer(buffer.Array);

		MemoryStream stream = new MemoryStream(msgContents);
		return ReadMessage(stream, int.MaxValue);
	}

	public override Message ReadMessage(Stream stream, int maxSizeOfHeaders, string contentType)
	{
		XmlDocument doc = new XmlDocument();
		doc.Load(stream);

		var securityNode = doc.DocumentElement.SelectSingleNode("//*[ local-name() = 'Security' and namespace-uri() = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd' ]");
		if (securityNode != null)
		{
			securityNode.ParentNode.RemoveChild(securityNode);
		}

		var payload = doc.OuterXml;

		var ms = new MemoryStream();
		var writer = new StreamWriter(ms);
		writer.Write(payload);
		writer.Flush();
		ms.Position = 0;

		var reader = XmlReader.Create(ms);
		return Message.CreateMessage(reader, int.MaxValue, this.MessageVersion);
	}

	public override ArraySegment<byte> WriteMessage(Message message, int maxMessageSize, BufferManager bufferManager, int messageOffset)
	{
		return innerEncoder.WriteMessage(message, maxMessageSize, bufferManager, messageOffset);
	}

	public override void WriteMessage(Message message, Stream stream)
	{
		innerEncoder.WriteMessage(message, stream);
	}

	public override bool IsContentTypeSupported(string contentType)
	{
		if (base.IsContentTypeSupported(contentType))
		{
			return true;
		}
		if (contentType.Length == this.MediaType.Length)
		{
			return contentType.Equals(this.MediaType, StringComparison.OrdinalIgnoreCase);
		}
		else
		{
			if (contentType.StartsWith(this.MediaType, StringComparison.OrdinalIgnoreCase)
				&& (contentType[this.MediaType.Length] == ';'))
			{
				return true;
			}
		}
		return false;
	}
}